@@ -13,8 +13,8 @@
 package Net::STOMP::Client;
 use strict;
 use warnings;
-our $VERSION  = "1.1";
-our $REVISION = sprintf("%d.%02d", q$Revision: 1.85 $ =~ /(\d+)\.(\d+)/);
+our $VERSION  = "1.2";
+our $REVISION = sprintf("%d.%02d", q$Revision: 1.91 $ =~ /(\d+)\.(\d+)/);
 
 #
 # used modules
@@ -87,8 +87,8 @@ sub _timeout : method {
 	};
     }
     unless ($operation =~ /^(connect|connected|receive|send)$/) {
-	Net::STOMP::Client::Error::repor("%s: unexpected timeout operation: %s",
-					 "Net::STOMP::Client", $operation);
+	Net::STOMP::Client::Error::report("%s: unexpected timeout operation: %s",
+					  "Net::STOMP::Client", $operation);
 	return();
     }
     return($timeout->{$operation});
@@ -318,6 +318,7 @@ sub new : method {
     $self->_serial(0);
     $self->_receipts({});
     $self->callbacks({});
+    $self->session("");
     return($self);
 }
 
@@ -363,8 +364,7 @@ sub queue_frame : method {
 	$receipt = $frame->header("receipt");
 	$self->_receipts()->{$receipt}++ if $receipt;
 	# encode the frame
-	$frame->debug(" sending") if $Net::STOMP::Client::Debug::Flags;
-	$data = $frame->_encode(version => $version);
+	$data = $frame->encode(version => $version);
     } else {
 	# handle the special NOOP frame (= newline) or an already encoded frame
 	$data = \$frame;
@@ -447,7 +447,7 @@ sub receive_frame : method {
     $version = $self->version();
     $bufref = $self->_io()->incoming_buffer_reference();
     $state = {};
-    $frame = Net::STOMP::Client::Frame::_decode($bufref,
+    $frame = Net::STOMP::Client::Frame::decode($bufref,
         version => $version,
 	state   => $state,
     );
@@ -469,7 +469,7 @@ sub receive_frame : method {
 	$result = $self->receive_data($timeout);
 	return($result) unless $result;
 	# do we have a complete frame now?
-	$frame = Net::STOMP::Client::Frame::_decode($bufref,
+	$frame = Net::STOMP::Client::Frame::decode($bufref,
 	    version => $version,
 	    state   => $state,
 	);
@@ -481,7 +481,6 @@ sub receive_frame : method {
 	direction => FLAG_DIRECTION_S2C,
     ) or return();
     # so far so good
-    $frame->debug(" received") if $Net::STOMP::Client::Debug::Flags;
     return($frame);
 }
 
@@ -725,9 +724,12 @@ sub _default_connected_callback ($$) {
 	$self->client_heart_beat(0);
 	$self->server_heart_beat(0);
     }
-    # keep track of session id too
+    # keep track of session id
     unless ($self->session()) {
-	$self->session($frame->header("session"));
+	$value = $frame->header("session");
+	# the session header is optional so we forge our own if it is missing
+	$value ||= sprintf("sid-%s", $self->_id());
+	$self->session($value);
 	return($self);
     }
     _unexpected_frame($frame, $frame->header("session"));
@@ -880,6 +882,7 @@ sub connect : method {
 	callback => sub { return($self->session()) },
 	timeout  => $self->_timeout("connected"),
     );
+    return() unless defined($session);
     return($self) if $session;
     Net::STOMP::Client::Error::report("Net::STOMP::Client->connect(): %s",
 				      "no CONNECTED frame received");
@@ -953,17 +956,18 @@ sub unsubscribe : method {
 
 sub send : method {
     my($self, %option) = @_;
-    my($frame, $timeout, $body);
+    my($frame, $timeout, %frameopt);
 
     $self->_check_invocation(scalar(@_)) or return();
     $timeout = delete($option{timeout});
-    $body = delete($option{body});
+    $frameopt{body} = delete($option{body})
+	if defined($option{body});
+    $frameopt{body_reference} = delete($option{body_reference})
+	if defined($option{body_reference});
+    $frameopt{command} = "SEND";
+    $frameopt{headers} = \%option;
     # send a SEND frame
-    $frame = Net::STOMP::Client::Frame->new(
-	command => "SEND",
-	headers => \%option,
-    );
-    $frame->body($body) if defined($body);
+    $frame = Net::STOMP::Client::Frame->new(%frameopt);
     $self->send_frame($frame, $timeout) or return();
     return($self);
 }
@@ -1378,7 +1382,8 @@ Some methods also support other options:
 
 =item send()
 
-C<body>: holds the body of the message to be sent
+C<body> or C<body_reference>: holds the body or body reference of the
+message to be sent
 
 =item ack()
 
@@ -1414,7 +1419,7 @@ return the file handle of the socket connecting the client and the server
 
 =item session()
 
-return the session identifier if connected or undef otherwise
+return the session identifier if connected or the empty string otherwise
 
 =item version()
 
@@ -1676,14 +1681,14 @@ positive number (meaning block at most this number of seconds).
 
 =head1 COMPATIBILITY
 
-This module implements the versions C<1.0> (see
-L<http://stomp.github.com/stomp-specification-1.0.html>) and C<1.1>
-(see L<http://stomp.github.com/stomp-specification-1.1.html>) of the
-protocol as well as well known extensions for JMS, AMQP, ActiveMQ and
-RabbitMQ.
+This module implements the versions
+C<1.0> (see L<http://stomp.github.com/stomp-specification-1.0.html>) and
+C<1.1> (see L<http://stomp.github.com/stomp-specification-1.1.html>)
+of the protocol as well as well known extensions for JMS, ActiveMQ,
+Apollo and RabbitMQ.
 
-It has been successfully tested against both ActiveMQ and RabbitMQ
-brokers.
+It has been successfully tested against ActiveMQ, Apollo, HornetQ and
+RabbitMQ brokers.
 
 =head1 SEE ALSO
 
